#include <visa.h>

/*= Useful Macros ===========================================================*/

/* - Basic Macros ---------------------------------------------------------- */

/* Error checking macro */
#define CHECKERR(fCal) if (rsrsc_status = (fCal), rsrsc_status < VI_SUCCESS) \
    return rsrsc_status

/* Error checking macro with jump to the ERROR: label */
#define GOCHECKERR(fCal) if (rsrsc_status = (fCal), rsrsc_status < VI_SUCCESS) \
                            goto ERROR_LABEL
/* Parameter checking macro */
#define CHECKPARAM(fCal,param) if (fCal != VI_SUCCESS) return VI_ERROR_PARAMETER##param

/* Checking path parameter */
#define CHECKATTENUATOR if (rsrsc_invalidViInt32Range (attenuator, 1, 7) != 0) return VI_ERROR_PARAMETER2;

/* Initialize default function parameters */
#define INIT\
    ViStatus            rsrsc_status = VI_SUCCESS; \
    rsrsc_instrRange    instrPtr = NULL; \
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));\
    if (instrPtr == NULL)\
        return VI_ERROR_PARAMETER1

/* Close function and return */
#define RETURN\
    return rsrsc_status;

/* - Additional Macros ----------------------------------------------------- */

/* Remap parameter error code */
#define REMAP(par_in, par_out) if (rsrsc_status == VI_ERROR_PARAMETER##par_in)\
                                    rsrsc_status = VI_ERROR_PARAMETER##par_out;\
                                if (rsrsc_status < VI_SUCCESS)\
                                    return rsrsc_status

#define REMAPINSTR(par_in, par_out) if (rsrsc_status == VI_ERROR_PARAMETER##par_in)\
                                    rsrsc_status = VI_ERROR_INSTR_PARAMETER##par_out;\
                                if (rsrsc_status < VI_SUCCESS)\
                                    return rsrsc_status

/* Check NULL pointer */
#define CHECKNULL(pointer, param) if (pointer == NULL) return VI_ERROR_PARAMETER##param


/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rsrsc_stringValPair is used in the rsrsc_error_message function           */
/* rsrsc_statusDataRanges is used to track session dependent status & ranges */
/*===========================================================================*/
typedef struct  rsrsc_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rsrsc_tStringValPair;

struct rsrsc_statusDataRanges
{
    ViChar              instrDriverRevision[256];
    ViBoolean           errorChecking;
};

typedef struct rsrsc_statusDataRanges *rsrsc_instrRange;
ViBoolean rsrsc_invalidViBooleanRange (ViBoolean val);
ViBoolean rsrsc_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rsrsc_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean rsrsc_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus rsrsc_checkStatus (ViSession instrSession);
ViStatus rsrsc_waitOPC (ViSession instrSession, ViPString command,
                        ViPString instrResp, ViInt32 timeout);
ViStatus rsrsc_sendCmd (ViSession instrSession, ViString cmd);
ViStatus rsrsc_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsrsc_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsrsc_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode);
ViStatus rsrsc_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode);
ViStatus rsrsc_setViBooleanAtten (ViSession instrSession,
                           ViString cmd,
                           ViInt32 attenuator,
                           ViBoolean value,
                           ViStatus parameterErrorCode);
ViStatus rsrsc_getViBooleanAtten (ViSession instrSession,
                           ViString cmd,
                           ViInt32 attenuator,
                           ViBoolean *value,
                           ViStatus parameterErrorCode);
ViStatus rsrsc_setViReal64 (ViSession vi,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode);
ViStatus rsrsc_setViReal64Atten (ViSession instrSession,
                            ViString cmd,
                            ViInt32  attenuator,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode);
ViStatus rsrsc_getViReal64Atten (ViSession instrSession,
                            ViString cmd,
                            ViInt32 attenuator,
                            ViReal64 *value,
                            ViStatus parameterErrorCode);
ViStatus rsrsc_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode);
ViStatus rsrsc_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsrsc_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViStatus parameterErrorCode);
ViStatus rsrsc_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsrsc_getViStringQuoted (ViSession instrSession,
                                  ViString cmd,
                                  ViString quotes,
                                  ViInt32  arraySize,
                                  ViChar   value[],
                                  ViStatus parameterErrorValue);
ViStatus rsrsc_setViStringAtten (ViSession instrSession,
                            ViString cmd,
                            ViInt32 path,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsrsc_setEnumAtten (ViSession instrSession,
                        ViString cmd,
                        ViInt32 attenuator,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsrsc_getEnumAtten (ViSession instrSession,
                        ViString cmd,
                        ViInt32 attenuator,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsrsc_checkIndexInList (ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError);
ViStatus rsrsc_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index);
ViStatus rsrsc_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count);
